from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import authenticate
from django import forms
from django.core.exceptions import ValidationError

from evotecore import widgets

from .models import User


class EvoteAuthenticationForm(forms.Form):
    """
    Base class for authenticating users. Extend this to get a form that accepts
    username/password logins.
    """
    email = forms.EmailField(
        label=_("Email"), max_length=254,
        widget=forms.EmailInput(
            attrs={'autofocus': True,
                   'placeholder': _("Email"),
                   'autocomplete': 'off'}))
    password = forms.CharField(
        label=_("Password"), strip=False,
        widget=forms.PasswordInput(
            attrs={'placeholder': _("Password"),
                   'autocomplete': 'off'}))

    error_messages = {
        'invalid_login': _(
            "Please enter a correct email and password. Note that both "
            "fields may be case-sensitive."
        ),
        'inactive': _("This account is inactive."),
    }

    def __init__(self, *args, **kwargs):
        """ """
        self.user_cache = None
        super(EvoteAuthenticationForm, self).__init__(*args, **kwargs)

    def clean(self):
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')
        if email is not None and password:
            self.user_cache = authenticate(email=email, password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login'
                )
            else:
                self.confirm_login_allowed(
                    self.user_cache, cleaned_data=self.cleaned_data)
        return self.cleaned_data

    def confirm_login_allowed(self, user, cleaned_data=None):
        """
        Controls whether the given User may log in. This is a policy setting,
        independent of end-user authentication. This default behavior is to
        allow login by active users, and reject login by inactive users.
        If the given user cannot log in, this method should raise a
        ``forms.ValidationError``.
        If the given user may log in, this method should return None.
        """
        if not user.is_active:
            raise forms.ValidationError(
                self.error_messages['inactive'],
                code='inactive',
            )

    def get_user_id(self):
        if self.user_cache:
            return self.user_cache.id
        return None

    def get_user(self):
        return self.user_cache


class PersonalDataForm(forms.ModelForm):
    """ """

    class Meta:
        model = User
        fields = ('photo', 'name', 'email', 'primary_phone',
                  'notice_method', 'notice_phone', 'notice_email')
        widgets = {
            'notice_method': widgets.Select
        }

    def __init__(self, *args, **kwargs):
        super(PersonalDataForm, self).__init__(*args, **kwargs)
        self.fields['photo'].widget.attrs['readonly'] = True
        self.fields['name'].widget.attrs['readonly'] = True
        self.fields['email'].widget.attrs['readonly'] = True
        self.fields['primary_phone'].widget.attrs['readonly'] = True

    def clean(self):
        super(PersonalDataForm, self).clean()
        notice_method = self.cleaned_data.get('notice_method', None)
        if notice_method == User.NoticeMethod.SMS:
            notice_phone = self.cleaned_data.get('notice_phone', None)
            if not notice_phone:
                self.add_error('notice_method', ValidationError(
                    _('You must have a notice phone to select SMS method.')))
        else:
            notice_email = self.cleaned_data.get('notice_email', None)
            if not notice_email:
                self.add_error('notice_method', ValidationError(
                    _('You must have a notice email to select Email method.')))

    def clean_photo(self):
        return self.instance.photo

    def clean_name(self):
        return self.instance.name

    def clean_email(self):
        return self.instance.email

    def clean_primary_phone(self):
        return self.instance.primary_phone
