from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from django.apps import apps
from django.db import transaction

from phonenumber_field.validators import validate_international_phonenumber
from evotecore.importers import BaseImporter


class ElectoralRollImporter(BaseImporter):
    """ """
    field_names = ('personnel_number', 'name', 'email', 'phone', 'category')
    BaseImporter.error_messages.update({
        'unique_personnel_number': _("Personnel number most be unique"),
        'unique_email': _("Email most be unique"),
        'invalid_email': _("Email invalid format"),
        'invalid_phone': _("Phone invalid format"),
        'invalid_category': _("Category not available"),
    })

    def __init__(self, election, delimiter=None):
        """ """
        super(ElectoralRollImporter, self).__init__(delimiter)
        self.election = election
        self.category_slugs = self.election.categories\
            .values_list('slug', flat=True)

    @transaction.atomic
    def run(self, document):
        """ """
        data = self.get_csv_data(document, True)
        ElectionVoter = apps.get_model("evotevoters.ElectionVoter")  # noqa
        return ElectionVoter.objects.bulk_import(self.election, data)

    def get_validator(self):
        """ """
        validator = super(ElectoralRollImporter, self).get_validator()
        validator.add_unique_check(
            'personnel_number',
            message=self.error_messages['unique_personnel_number'])
        validator.add_unique_check(
            'email', message=self.error_messages['unique_email'])
        validator.add_value_predicate(
            'email', self._check_email,
            message=self.error_messages['invalid_email'])
        validator.add_value_predicate(
            'phone', self._check_phone,
            message=self.error_messages['invalid_phone'])
        validator.add_value_predicate(
            'category', self._check_category,
            message=self.error_messages['invalid_category'])
        return validator

    def _check_category(self, value):
        """ """
        return value in self.category_slugs

    def _check_email(self, value):
        try:
            validate_email(value)
        except ValidationError:
            return False
        return True

    def _check_phone(self, value):
        """ """
        try:
            validate_international_phonenumber(value)
        except ValidationError:
            return False
        return True


class CandidateImporter(BaseImporter):
    """ """
    field_names = ('personnel_number', 'designation')
    BaseImporter.error_messages.update({
        'unique_personnel_number': _("Personnel number most be unique")
    })

    def __init__(self, election, delimiter=None):
        """ """
        super(CandidateImporter, self).__init__(delimiter)
        self.election = election

    @transaction.atomic
    def run(self, document):
        """ """
        data = self.get_csv_data(document, True)
        Candidate = apps.get_model("evoteelections.Candidate")  # noqa
        return Candidate.objects.bulk_import(self.election, data)

    def get_validator(self):
        """ """
        validator = super(CandidateImporter, self).get_validator()
        validator.add_unique_check(
            'personnel_number',
            message=self.error_messages['unique_personnel_number'])
        return validator
