from __future__ import unicode_literals

from cStringIO import StringIO
from csvvalidator import VALUE_PREDICATE_FALSE, UNIQUE_CHECK_FAILED

from django.test import TestCase

from evotevotings.elections.tests.utils import ElectionsTestUtils
from evotevotings.elections.importers import (ElectoralRollImporter,
                                              CandidateImporter)


class ElectoralRollImporterTestCase(ElectionsTestUtils, TestCase):
    """ """
    def setUp(self):
        """ """
        election = self.create_election()
        self.electoral_roll_importer = ElectoralRollImporter(election)
        self.header = "personnel_number;name;email;phone;category\n"

    def _get_csv(self, lines=None):
        """ """
        csv = self.header
        if lines:
            csv += "\n".join(lines)
        return StringIO(csv)

    def test_validate_with_valid_header_fields(self):
        """ """
        csv = self._get_csv()
        result = self.electoral_roll_importer.validate(csv)
        self.assertEqual(result, [])

    def test_validate_with_invalid_header_fields(self):
        """ """
        invalid_csv_header = StringIO("personnel_number;name;phones;category")
        result = self.electoral_roll_importer.validate(invalid_csv_header)
        self.assertEqual(result[0]['unexpected'], set(["phones"]))
        self.assertEqual(result[0]['missing'], set(["phone", "email"]))

    def test_validate_unique_email(self):
        """ """
        csv = self._get_csv([
            "123;test name;dev@eurotux.com;+351933456789;one",
            "124;test name;dev@eurotux.com;+351933456789;one"
        ])
        result = self.electoral_roll_importer.validate(csv)
        self.assertEqual(result[0]['code'], UNIQUE_CHECK_FAILED)
        self.assertEqual(result[0]['key'], "email")

    def test_validate_unique_personnel_number(self):
        """ """
        csv = self._get_csv([
            "123;test name;dev1@eurotux.com;+351933456789;one",
            "123;test name;dev2@eurotux.com;+351933456789;one"
        ])
        result = self.electoral_roll_importer.validate(csv)
        self.assertEqual(result[0]['code'], UNIQUE_CHECK_FAILED)
        self.assertEqual(result[0]['key'], "personnel_number")

    def test_validate_category(self):
        """ """
        csv = self._get_csv([
            "123;test name;dev@eurotux.com;+351933456789;two",
        ])
        result = self.electoral_roll_importer.validate(csv)
        self.assertEqual(result[0]['code'], VALUE_PREDICATE_FALSE)
        self.assertEqual(result[0]['field'], "category")

    def test_validate_email_format(self):
        """ """
        csv = self._get_csv([
            "123;test name;dev@eurotux.com;+351933456789;one",
            "123;test name;dev@eurotux;+351933456789;one",
        ])
        result = self.electoral_roll_importer.validate(csv)
        self.assertEqual(result[0]['code'], VALUE_PREDICATE_FALSE)
        self.assertEqual(result[0]['field'], "email")

    def test_validate_phone_format(self):
        """ """
        csv = self._get_csv([
            "123;test name;dev@eurotux.com;123456789;one",
        ])
        result = self.electoral_roll_importer.validate(csv)
        self.assertEqual(result[0]['code'], VALUE_PREDICATE_FALSE)
        self.assertEqual(result[0]['field'], "phone")

    def test_run(self):
        csv = self._get_csv([
            "1;test name;dev1@eurotux.com;+351933456789;one",
            "2;test name;dev2@eurotux.com;+351933456789;one",
        ])
        voters = self.electoral_roll_importer.run(csv)
        self.assertEqual(len(voters), 2)


class CandidateImporterTestCase(ElectionsTestUtils, TestCase):
    """ """

    def setUp(self):
        """ """
        self.election = self.create_election()
        self.candidate_importer = CandidateImporter(self.election)
        self.header = "personnel_number;designation\n"

    def _get_csv(self, lines=None):
        """ """
        csv = self.header
        if lines:
            csv += "\n".join(lines)
        return StringIO(csv)

    def test_validate_with_valid_header_fields(self):
        """ """
        csv = self._get_csv()
        result = self.candidate_importer.validate(csv)
        self.assertEqual(result, [])

    def test_validate_with_invalid_header_fields(self):
        """ """
        invalid_csv_header = StringIO("personnel_number;designations")
        result = self.candidate_importer.validate(invalid_csv_header)
        self.assertEqual(result[0]['unexpected'], set(["designations"]))
        self.assertEqual(result[0]['missing'], set(["designation"]))
        self.assertEqual(result[0]['message'], "Invalid header")

    def test_validate_with_invalid_record_length(self):
        """ """
        invalid_csv = self._get_csv([
            "123;test candidate;Test",
        ])
        result = self.candidate_importer.validate(invalid_csv)
        self.assertEqual(result[0]['message'], "Invalid record length")

    def test_validate_unique_personnel_number(self):
        """ """
        csv = self._get_csv([
            "123;test candidate",
            "123;test candidate"
        ])
        result = self.candidate_importer.validate(csv)
        self.assertEqual(result[0]['code'], UNIQUE_CHECK_FAILED)
        self.assertEqual(result[0]['key'], "personnel_number")
        self.assertEqual(
            result[0]['message'], "Personnel number most be unique")

    def test_run(self):
        csv = self._get_csv([
            "1234;test candidate",
            "1235;test candidate 2",
        ])
        imported = self.candidate_importer.run(csv)
        self.assertEqual(len(imported), 2)
