from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
from django.views import View
from django.http import JsonResponse

from evotevotings.processes.models import (ElectoralProcess, ElectoralAction,
                                           ElectoralMember)
from evoteajaxforms.views import FormPartialJsonView, AjaxResponseAction
from evotevotings.management.forms import ElectoralMemberEmailForm

from .forms import ActionValidationKeysForm, ElectoralMemberForm
from .registers import action_validation


@method_decorator(login_required, name="dispatch")
class ActionValidationAjaxView(FormPartialJsonView):
    """ """
    template_name = "management/partials/action_validate.html"
    form_class = ActionValidationKeysForm
    action_validation = None

    def get_form_kwargs(self):
        """ """
        electoral_action = get_object_or_404(
            ElectoralAction, pk=self.kwargs.get("pk", None),
            electoral_process=self.kwargs.get("process_pk"))
        self.action_validation = self.kwargs.get("pk")
        electoral_process = electoral_action.electoral_process
        kwargs = super(ActionValidationAjaxView, self).get_form_kwargs()
        kwargs.update({
            'num_members': electoral_process.electoral_members.count(),
            'strength': electoral_action.strength
        })
        return kwargs

    def form_valid(self, form):
        action_validation.register(
            self.request, self.kwargs.get("process_pk"),
            self.action_validation)
        return super(ActionValidationAjaxView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ManageActionValidationAjaxView(ActionValidationAjaxView):
    """ """
    action_validation = "manage"

    def get_form_kwargs(self):
        """ """
        electoral_process = get_object_or_404(
            ElectoralProcess, pk=self.kwargs.get("process_pk"))
        kwargs = super(ActionValidationAjaxView, self).get_form_kwargs()
        kwargs.update({
            'num_members': electoral_process.electoral_members.count(),
            'strength': ElectoralAction.Strength.CRITICAL
        })
        return kwargs


@method_decorator(login_required, name="dispatch")
class ElectoralMemberAddFormView(FormPartialJsonView):
    """ """
    model = ElectoralMember
    form_class = ElectoralMemberForm
    template_name = "management/partials/electoralmember_form.html"
    action = AjaxResponseAction.REFRESH

    def form_valid(self, form):
        """ """
        self.electoral_process = get_object_or_404(
            ElectoralProcess, pk=self.kwargs['process_pk'])
        member, created = ElectoralMember.objects.update_or_create_member(
            email=form.cleaned_data.get('email'),
            name=form.cleaned_data.get('name'),
            electoral_process=self.electoral_process,
            designation=form.cleaned_data.get('designation'),
            role=form.cleaned_data.get('role'),
            phone=form.cleaned_data.get('phone'))

        if created:
            messages.success(self.request, _("Member added successfully"))
        else:
            messages.warning(self.request, _("User is a member already"))
        return self.json_to_response()

    def form_invalid(self, form):
        """ """
        messages.success(
            self.request, _("There was an error adding the member"))
        return super(ElectoralMemberAddFormView, self).form_invalid(form)

    def get_form_kwargs(self):
        form_kwargs = super(ElectoralMemberAddFormView, self).get_form_kwargs()
        form_kwargs.update({
            'electoral_process_pk': self.kwargs['process_pk']
        })
        return form_kwargs


@method_decorator(login_required, name="dispatch")
class ElectoralMemberEditFormView(FormPartialJsonView):
    """ """
    model = ElectoralMember
    form_class = ElectoralMemberForm
    template_name = "management/partials/electoralmember_form.html"
    action = AjaxResponseAction.REFRESH

    def get_form_kwargs(self):
        form_kwargs = super(ElectoralMemberEditFormView, self)\
            .get_form_kwargs()
        form_kwargs.update({
            'instance': ElectoralMember.objects.get(pk=self.kwargs['pk']),
            'electoral_process_pk': self.kwargs['process_pk']
        })
        return form_kwargs

    def form_valid(self, form):
        """ """
        messages.success(self.request, _("Member updated successfully"))
        return super(ElectoralMemberEditFormView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ElectoralMemberVerifyView(View):
    """ """

    def get(self, request, *args, **kwargs):
        form = ElectoralMemberEmailForm(request.GET)
        response_data = {
            'error': 'User not found'
        }
        if form.is_valid():
            user = form.get_user()

            if user:
                response_data = {
                    'name': user['name'],
                    'phone': str(user['phone']),
                    'enable': ['role', 'designation']
                }

        return JsonResponse(response_data, status=200)
