from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.template.loader import get_template
from django.utils.html import format_html
from django.template import Context

import django_tables2 as tables
from evotevotings.processes.models import ElectoralProcess, ElectoralMember
from evotevotings.elections.models import Election, ElectoralRoll
from evotevotings.voters.models import ElectionVoter
from django_tables2.utils import AttributeDict
from evotecore.tables import ImageColumn, DateTimeColumn, CounterColumn


class BaseProcessTable(tables.Table):
    """
    Base table with common fields
    """
    logo = ImageColumn('', attrs={'th': {'class': 's-10'}})
    identifier = tables.Column(attrs={'th': {'class': 's-15'}})
    description = tables.Column(attrs={'th': {'class': 's-50'}})

    class Meta:
        model = ElectoralProcess
        empty_text = _(u'No electoral processes.')
        orderable = False
        attrs = {"class": "pure-table pure-table-line"}
        fields = ('logo', 'identifier', 'description')


class ElectoralProcessMemberTable(BaseProcessTable):
    """
    Table for Electoral Member
    """
    select_button = tables.LinkColumn(
        viewname='management:electoralprocess-detail',
        args=[tables.A('slug')],
        orderable=False,
        attrs={'a':
               {'class': 'pure-button pure-button-primary pure-button-round' +
                ' pull-right'},
               'th': {'class': 's-10'}},
        text=_('Select'),
        verbose_name=''
    )

    class Meta(BaseProcessTable.Meta):
        fields = BaseProcessTable.Meta.fields + ('select_button',)


class ElectoralProcessInstitutionalTable(BaseProcessTable):
    """
    Table for Institutional responsabile
    """
    select_button = tables.LinkColumn(
        viewname='management:electoralprocess-edit',
        args=[tables.A('slug')],
        orderable=False,
        attrs={'a':
               {'class': 'pure-button pure-button-primary pure-button-round' +
                ' pull-right'},
               'th': {'class': 's-10'}},
        text=_('Select'),
        verbose_name=''
    )

    class Meta(BaseProcessTable.Meta):
        fields = BaseProcessTable.Meta.fields + ('select_button',)


class ElectoralProcessDetailTable(BaseProcessTable):
    """
    Table for Electoral process detail
    """
    identifier = tables.Column(attrs={'th': {'class': 's-30'}})
    description = tables.Column(attrs={'th': {'class': 's-40'}})
    publication_date = DateTimeColumn(attrs={'th': {'class': 's-20'}})
    electoral_documents = tables.Column(attrs={'td': {'class': 'text-center'}})
    edit_button = tables.LinkColumn(
        viewname='management:electoralprocess-edit',
        args=[tables.A('slug')],
        text=format_html("<i class='ev-icon evi-edit'></i>"),
        verbose_name=''
    )
    elections = tables.Column(
        verbose_name=_('Number of elections'),
        attrs={'td': {'class': 'text-center'}})
    elections_button = tables.LinkColumn(
        viewname='management:election-list',
        args=[tables.A('slug')],
        attrs={
            'a': {
                'class': 'pure-button pure-button-primary pure-button-round' +
                ' pull-right'
            }
        },
        text=_('Elections'),
        verbose_name=''
    )

    class Meta(BaseProcessTable.Meta):
        fields = BaseProcessTable.Meta.fields + (
            'publication_date', 'electoral_documents', 'edit_button',
            'elections', 'elections_button')

    def render_electoral_documents(self, record):
        return record.electoral_documents.count()

    def render_elections(self, record):
        return record.elections.count()


class ElectionIconColumn(tables.Column):

    def render(self, filled=False, unlock=False, num=None):
        icon = "evi-b-lock"
        num = num if num is not None else ""
        if unlock:
            icon = "evi-b-success" if filled else "evi-noinfo"
        return format_html("<i class='ev-icon evi-circle {}'></i><br>{}",
                           icon, num)


class ElectionTable(tables.Table):
    """ """
    identifier = tables.LinkColumn(
        verbose_name=_("identifier"),
        viewname='management:election-detail-configs',
        kwargs={
            'slug': tables.A('electoral_process.slug'),
            'slug_election': tables.A('slug')
        }
    )
    operations_panel = tables.LinkColumn(
        verbose_name=_("operations panel"), text=_('Manage'),
        viewname='management:election-list',
        kwargs={'slug': tables.A('electoral_process.slug')},
        attrs={'a': {'class': 'pure-button pure-button-round'}}
    )
    configs = ElectionIconColumn(_("configurations"), empty_values=())
    calendar = ElectionIconColumn(_("calendar"), empty_values=())
    categories = ElectionIconColumn(_("categories"), empty_values=())
    electoral_rolls = ElectionIconColumn(_("electoral rolls"), empty_values=())
    candidates = ElectionIconColumn(_("candidates"), empty_values=())
    bulletin = ElectionIconColumn(_("bulletin"), empty_values=())
    edit_button = tables.LinkColumn(
        viewname='management:election-detail-configs',
        kwargs={
            'slug': tables.A('electoral_process.slug'),
            'slug_election': tables.A('slug')
        },
        text=format_html("<i class='ev-icon evi-edit'></i>"),
        verbose_name=''
    )

    class Meta:
        model = Election
        fields = ("identifier", "operations_panel", "configs", "calendar",
                  "categories", "electoral_rolls", "candidates", "bulletin",
                  "edit_button")
        orderable = False
        empty_text = _(u"No elections available.")
        attrs = {"class": "pure-table"}

    def render_operations_panel(self, column, value, record, bound_column):
        if record.can_manage_operations:
            column.attrs['a']['class'] += ' pure-button-disabled'
            return column.render_link("#", record=record, value=value)
        else:
            column.attrs['a']['class'] += ' pure-button-primary'
        return column.render(value, record, bound_column)

    def render_configs(self, column, record):
        return column.render(
            filled=record.configs_filled, unlock=record.can_change_configs())

    def render_calendar(self, column, record):
        return column.render(
            filled=record.calendar_filled, unlock=record.can_change_calendar())

    def render_categories(self, column, record):
        return column.render(
            filled=record.categories.exists(), num=record.categories.count(),
            unlock=record.can_change_categories())

    def render_electoral_rolls(self, column, record):
        return column.render(
            filled=record.election_voters.exists(),
            num=record.election_voters.count(),
            unlock=record.can_change_rolls())

    def render_candidates(self, column, record):
        return column.render(
            filled=record.candidates.exists(), num=record.candidates.count(),
            unlock=record.can_change_categories())

    def render_bulletin(self, column, record):
        return column.render(
            filled=record.bulletin_filled, unlock=record.can_change_bulletin())


class ElectoralMemberSimpleTable(tables.Table):
    """
    Table for Electoral Members detail
    """
    counter = CounterColumn(empty_values=(), verbose_name='#')
    name = tables.Column(accessor='user.name', attrs={'th': {'class': 's-20'}})
    designation = tables.Column(attrs={'th': {'class': 's-20'}})
    email = tables.Column(accessor='user.email')

    class Meta:
        model = ElectoralMember
        empty_text = _(u'No electoral members.')
        orderable = False
        attrs = {"class": "pure-table pure-table-striped"}
        fields = ('counter', 'name', 'role', 'designation', 'email', 'phone')


class ElectoralMemberTable(ElectoralMemberSimpleTable):
    """
    Table for New Electoral Member
    """
    edit = tables.Column(empty_values=(), verbose_name='')
    delete = tables.LinkColumn(
        viewname='management:electoralmember-remove',
        kwargs={
            'process_pk': tables.A('electoral_process_id'),
            'pk': tables.A('pk')
        },
        attrs={
            'a': {
                'data-remodal-target': 'modalConfirmOP'
            }
        },
        text=format_html("<i class='ev-icon evi-delete'></i>"),
        verbose_name=''
    )

    class Meta(ElectoralMemberSimpleTable.Meta):
        fields = ElectoralMemberSimpleTable.Meta.fields + ('edit', 'delete')

    def render_edit(self, record):
        url = reverse_lazy(
            "management:electoralmember-edit",
            args=[record.electoral_process_id, record.pk])
        return format_html("<a href='#' data-partial-url='{}'>" +
                           "<i class='ev-icon evi-edit'></i></a>", url)


class ElectoralRollTable(tables.Table):
    """Table to list electoral rolls"""
    designation = tables.LinkColumn(
        viewname='management:electoralroll-detail',
        kwargs={'slug': tables.A('slug')},
        accessor="document.designation"
    )
    imported = tables.BooleanColumn(default=_("Importing..."))
    published_on = tables.Column(default=_("Never published"))
    actions = tables.Column(verbose_name="", empty_values=(), attrs={
        "td": {"class": "text-right"}
    })

    class Meta:
        model = ElectoralRoll
        empty_text = _(u'No electoral rolls imported yet...')
        attrs = {"class": "pure-table pure-table-alt"}
        fields = ('designation', 'version', 'published_on', 'imported',
                  'actions')
        orderable = False

    def render_actions(self, record, table):
        """ """
        published = ElectoralRoll.objects.get_published(record.election_id)
        if published and published.pk == record.pk:
            attrs = {
                "href": "#",
                "disabled": "",
                "class": "pure-button pure-button-secondary pure-button-round"
            }
            return format_html(
                "<a {}>{}</a>",
                AttributeDict(attrs).as_html(),
                _("Published electoral roll")
            )
        # render form to publish electoralroll
        to_publish = ElectoralRoll.objects.get_to_publish(record.election_id)
        if to_publish and to_publish.pk == record.pk:
            context = getattr(table, 'context', Context())
            context.update({"object": record})
            return get_template(
                "management/includes/electoralroll_publish_form.html")\
                .render(context)
        return ""


class ElectionVoterTable(tables.Table):
    """
    Table for Electoral Members detail
    """

    class Meta:
        model = ElectionVoter
        fields = ('name', 'personnel_number', 'category')
        attrs = {"class": "pure-table pure-table-striped"}
        orderable = False
        empty_text = _(u'No voters to show.')
