from __future__ import unicode_literals

from StringIO import StringIO
import json

from django.core.urlresolvers import reverse

from evotevotings.processes.tests.utils import ProcessesTestUtils
from evotevotings.processes.utils import prepopulate_electoralactions
from evotevotings.processes.models import ElectoralMember
from evoteajaxforms.views import AjaxResponseAction

from. import ManagementTestCase


class ActionValidationAjaxViewTest(ManagementTestCase, ProcessesTestUtils):
    """ """

    fixture = ['actions.json']

    def setUp(self):
        # prepare a logged in member
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.create_electoral_member(**self.credentials)
        self.client.login(**self.credentials)
        # prepare process and prepopulate ElectoralAction's.
        self.process = self.create_electoral_process()
        prepopulate_electoralactions(self.process)
        self.url = reverse("management:action-validate", kwargs={
            "process_pk": self.process.pk,
            "pk": 1
        })

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/partials/action_validate.html")

    def test_get_unexistent_action(self):
        """ """
        url = reverse("management:action-validate", kwargs={
            "process_pk": self.process.pk,
            "pk": 100
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_unexistent_process(self):
        """ """
        url = reverse("management:action-validate", kwargs={
            "process_pk": 3,
            "pk": 1
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_post(self):
        """ """
        file = StringIO(b"Dummy file")
        data = {'key1': file}
        response = self.client.post(self.url, data=data)
        self.assertEqual(response.status_code, 200)
        response_json = json.loads(response.content)
        self.assertEqual(response_json['action'], AjaxResponseAction.NOTHING)


class ManageActionValidationAjaxViewTest(
        ManagementTestCase, ProcessesTestUtils):
    """ """

    fixture = ['actions.json']

    def setUp(self):
        # prepare a logged in member
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.create_electoral_member(**self.credentials)
        self.client.login(**self.credentials)
        # prepare process
        process = self.create_electoral_process()
        self.url = reverse("management:manageaction-validate", kwargs={
            "process_pk": process.pk,
        })

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/partials/action_validate.html")

    def test_get_unexistent_process(self):
        """ """
        url = reverse("management:manageaction-validate", kwargs={
            "process_pk": 3,
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_post(self):
        """ """
        file = StringIO(b"Dummy file")
        data = {"action-key1": file}
        response = self.client.post(self.url, data=data)
        self.assertEqual(response.status_code, 200)
        response_json = json.loads(response.content)
        self.assertEqual(response_json['action'], AjaxResponseAction.NOTHING)


class ElectoralMemberAddFormViewTest(
        ManagementTestCase, ProcessesTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process()
        self.url = reverse("management:electoralmember-add", kwargs={
            "process_pk": self.process.pk
        })
        self.valid_data = {
            'email': 'tfb@eurotux.com',
            'name': 'Tiago Brito',
            'role': '0',
            'designation': 'test',
            'phone': '+351912233445'
        }

    def test_post_valid(self):
        """ """
        response = self.client.post(self.url, data=self.valid_data)
        created = ElectoralMember.objects.filter(
            user__email=self.valid_data['email']).exists()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(created, True)

    def test_post_invalid(self):
        """ """
        data = {
            'email': 'tfb@eurotux.com',
            'name': 'Tiago Brito',
            'role': '',
            'designation': '',
            'phone': '+351912233445'
        }
        response = self.client.post(self.url, data=data)
        created = ElectoralMember.objects.filter(
            user__email=data['email']).exists()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(created, False)

    def test_post_user_exists(self):
        """ """
        data = {
            'email': 'tfb@eurotux.com',
            'name': 'error',
            'role': '0',
            'designation': 'test',
            'phone': '+351912233445'
        }
        response = self.client.post(self.url, data=self.valid_data)
        created = ElectoralMember.objects.filter(
            user__email=data['email']).exists()
        duplicated = self.client.post(self.url, data=data)
        created_duplicated = ElectoralMember.objects.filter(
            user__name=data['name']).exists()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(created, True)
        self.assertEqual(duplicated.status_code, 200)
        self.assertEqual(created_duplicated, False)


class ElectoralMemberEditFormViewTest(
        ManagementTestCase, ProcessesTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.member = self.create_electoral_member()
        self.url = reverse("management:electoralmember-edit", kwargs={
            "process_pk": self.member.electoral_process.pk,
            "pk": self.member.pk
        })

    def test_post_valid(self):
        """ """
        data = {
            'email': self.member.user.email,
            'name': self.member.user.name,
            'role': self.member.role,
            'designation': 'CHANGED',
            'phone': self.member.phone
        }
        response = self.client.post(self.url, data=data)
        updated = ElectoralMember.objects.filter(
            designation=data['designation']).exists()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(updated, True)


class ElectoralMemberVerifyViewTest(
        ManagementTestCase, ProcessesTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.url = reverse("management:electoralmember-verify") +\
            "?email=tfb@eurotux.com"

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        content = json.loads(response.content)
        result = False
        if 'name' in content and content['name'] == 'Tiago':
            result = True
        self.assertEqual(response.status_code, 200)
        self.assertEqual(result, True)
