from __future__ import unicode_literals

import json
from mock import patch, MagicMock

from django.forms.models import model_to_dict
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.conf import settings
from django.core.files import File

from evotevotings.elections.tests.utils import ElectionsTestUtils
from evotevotings.management.registers import action_validation
from evotevotings.management.forms import (ManagementAuthenticationForm,
                                           ElectoralProcessAsInstitutionalForm,
                                           ElectoralProcessAsMemberForm)
from evotevotings.management.tables import (
    ElectoralProcessInstitutionalTable,
    ElectoralProcessMemberTable)
from evotevotings.processes.utils import prepopulate_electoralactions
from evotevotings.processes.models import ElectoralMember, ElectoralProcess
from evoteajaxforms.views import AjaxResponseAction

from . import ManagementTestCase


class LoginViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.credentials = {
            "email": "dev@eurotux.com",
            "password": "password"
        }
        self.member = self.create_electoral_member(**self.credentials)
        self.url = settings.LOGIN_URL

    def test_login_view(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "management/login.html")

    def test_login_member(self):
        """ """
        self.credentials['profile'] = \
            ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER
        response = self.client.post(self.url, follow=True,
                                    data=self.credentials)
        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL)
        self.assertEqual(response.context[0]["user"], self.member.user)

    def test_login_member_with_wrong_credentials(self):
        self.credentials["password"] = "wrong_password"
        self.credentials['profile'] = \
            ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER
        response = self.client.post(self.url, data=self.credentials)
        error_message = \
            ManagementAuthenticationForm.error_messages['invalid_login']
        self.assertFormError(response, "form", None, error_message)


class LogoutViewTest(ManagementTestCase):

    def setUp(self):
        self.login_as_member()

    def test_logout_view(self):
        """ """
        response = self.client.get(reverse("management:logout"), follow=True)
        self.assertRedirects(response, settings.LOGIN_URL)
        self.assertIsInstance(response.context[0]["user"], AnonymousUser)


class ElectoralProcessListViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.url = reverse("management:electoralprocess-list")

    def test_used_template(self):
        """ """
        self.login_as_member()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_list.html")

    def test_used_table_as_institutional(self):
        self.login_as_institutional()
        response = self.client.get(self.url)
        table_used = isinstance(
            response.context['table'], ElectoralProcessInstitutionalTable)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(table_used, True)

    def test_used_table_as_member(self):
        self.login_as_member()
        response = self.client.get(self.url)
        table_used = isinstance(
            response.context['table'], ElectoralProcessMemberTable)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(table_used, True)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class DetailViewTest(ManagementTestCase):

    def setUp(self):
        self.login_as_member()
        self.process = self.create_electoral_process()
        self.url = reverse("management:electoralprocess-detail", kwargs={
            "slug": self.process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["object"].pk, self.process.pk)
        self.assertTemplateUsed(
            response, "management/electoralprocess_detail.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-detail", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectoralActionViewTest(ManagementTestCase):

    fixture = ['actions.json']

    def setUp(self):
        self.login_as_member()
        self.actions = self.create_actions(2)
        self.process = self.create_electoral_process()
        self.url = reverse("management:electoralprocess-actions", kwargs={
            "slug": self.process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["object"].pk, self.process.pk)
        self.assertTemplateUsed(
            response, "management/electoralprocess_actions.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_post_with_action_validated(self):
        data = {
            'electoral_actions-TOTAL_FORMS': '2',
            'electoral_actions-INITIAL_FORMS': '2',
            'electoral_actions-0-action': '1',
            'electoral_actions-0-id': '1',
            'electoral_actions-0-strength': '2',
            'electoral_actions-1-action': '2',
            'electoral_actions-1-id': '2',
            'electoral_actions-1-strength': '1',
        }
        # register manage action as validated
        self._register_action_validation("manage")
        prepopulate_electoralactions(self.process)
        response = self.client.post(self.url, data=data)
        electoral_actions = response.context['object'].electoral_actions.all()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(electoral_actions[0].strength, 2)
        self.assertEqual(electoral_actions[1].strength, 1)

    def test_post_with_action_not_validated(self):
        data = {
            'electoral_actions-TOTAL_FORMS': '1',
            'electoral_actions-INITIAL_FORMS': '1',
            'electoral_actions-0-action': '1',
            'electoral_actions-0-id': '1',
            'electoral_actions-0-strength': '2',
        }
        prepopulate_electoralactions(self.process)
        response = self.client.post(self.url, data=data)
        formset = response.context['formset']
        self.assertEqual(response.status_code, 200)
        self.assertFormsetError(response, "formset", None, None,
                                formset.error_messages['action_validation'])

    def _register_action_validation(self, action):
        key = action_validation.prefix_key(self.process.pk)
        session = self.client.session
        session[key] = action
        session.save()


class ElectionListViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election()
        self.url = reverse("management:election-list", kwargs={
            "slug": self.election.electoral_process.slug
        })

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "management/election_list.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_queryset_filtered_by_electoral_process(self):
        """ """
        election2 = self.create_election()
        response = self.client.get(self.url)
        qs = response.context[0]['object_list']
        self.assertIn(self.election, list(qs))
        self.assertNotIn(election2, list(qs))

    def test_view_extra_context_data(self):
        """ """
        response = self.client.get(self.url)
        self.assertIn('electoral_process', response.context[0])
        self.assertEqual(self.election.electoral_process_id,
                         response.context[0]['electoral_process'].pk)


class ElectoralProcessFormViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.url = reverse("management:electoralprocess-new")
        self.process = self.create_electoral_process()
        self.edit_url = reverse(
            "management:electoralprocess-edit", kwargs={
                "slug": self.process.slug
            })

    def test_used_template(self):
        """ """
        self.login_as_institutional()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_form.html")

    def test_used_form_as_institutional(self):
        self.login_as_institutional()
        response = self.client.get(self.url)
        form_used = isinstance(
            response.context['form'], ElectoralProcessAsInstitutionalForm)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(form_used, True)

    def test_used_form_as_member(self):
        url = reverse(
            "management:electoralprocess-edit", kwargs={
                "slug": self.process.slug
            })
        self.login_as_member()
        response = self.client.get(url)
        form_used = isinstance(
            response.context['form'], ElectoralProcessAsMemberForm)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(form_used, True)

    def test_post_valid(self):
        """ """
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
        }
        self.login_as_institutional()
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_post_valid_as_member(self):
        """ """
        file_mock = MagicMock(spec=File, name='FileMock')
        file_mock.name = 'test1.jpg'
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
            'process-publication_date': '29-11-2016 00:00',
            'electoral-documents-TOTAL_FORMS': 1,
            'electoral-documents-INITIAL_FORMS': 0,
            'electoral-documents-MIN_NUM_FORMS': 1,
            'electoral-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-designation': 'teste',
            'electoral-documents-0-documents-TOTAL_FORMS': 1,
            'electoral-documents-0-documents-INITIAL_FORMS': 0,
            'electoral-documents-0-documents-MIN_NUM_FORMS': 0,
            'electoral-documents-0-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-id': '',
            'electoral-documents-0-documents-0-designation': 'test',
            'electoral-documents-0-documents-0-id': '',
            'electoral-documents-0-documents-0-document': file_mock,

        }

        self.login_as_member()
        response = self.client.post(self.edit_url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_post_invalid(self):
        """ """
        data = {
            'id_process-identifier_pt': 'teste',
            'id_process-identifier_en': 'test',
            'id_process-description_pt': 'description in pt',
            'id_process-description_en': 'description in en',
        }
        self.login_as_institutional()
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_post_invalid_as_member(self):
        """ """
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
            'process-publication_date': '26-11-2016 00:00',
            'electoral-documents-TOTAL_FORMS': 1,
            'electoral-documents-INITIAL_FORMS': 1,
            'electoral-documents-MIN_NUM_FORMS': 1,
            'electoral-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-designation': '',
            'electoral-documents-0-documents-TOTAL_FORMS': 2,
            'electoral-documents-0-documents-INITIAL_FORMS': 1,
            'electoral-documents-0-documents-MIN_NUM_FORMS': 0,
            'electoral-documents-0-documents-MAX_NUM_FORMS': 1000,
            'electoral-documents-0-id': 1,
            'electoral-documents-0-documents-0-designation': 'test',
            'electoral-documents-0-documents-0-id': 1,
        }
        self.login_as_member()
        response = self.client.post(self.edit_url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectoralProcessUpdateViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process()
        self.election = self.create_election(electoral_process=self.process)
        self.url = reverse(
            "management:electoralprocess-edit", kwargs={
                "slug": self.process.slug
            })
        self.data_valid = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': 'description in pt',
            'process-description_en': 'description in en',
        }

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_form.html")

    def test_post_valid(self):
        """ """
        response = self.client.post(self.url, data=self.data_valid)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_post_valid_state_new(self):
        """ """
        process = self.create_electoral_process(state=0)
        url = reverse(
            "management:electoralprocess-edit", kwargs={
                "slug": process.slug
            })
        response = self.client.post(url, data=self.data_valid)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_post_formset_invalid(self):
        """ """
        data = {
            'process-identifier_pt': 'teste',
            'process-identifier_en': 'test',
            'process-description_pt': '',
            'process-description_en': '',
        }
        response = self.client.post(self.url, data=data)
        errors = False
        if 'errors_list' in response._container[0]:
            errors = True
        self.assertEqual(response.status_code, 200)
        self.assertEqual(errors, True)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectoralProcessElectionsViewTest(
        ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process()
        self.election = self.create_election(electoral_process=self.process)
        self.url = reverse(
            "management:electoralprocess-elections", kwargs={
                "slug": self.process.slug
            })
        self.data_valid = {
            'elections-TOTAL_FORMS': '1',
            'elections-INITIAL_FORMS': '0',
            'elections-MIN_NUM_FORMS': '1',
            'elections-MAX_NUM_FORMS': '1000',
            'elections-0-identifier_pt': 'election test in pt',
            'elections-0-identifier_en': 'election test in en',
            'elections-0-description_pt': 'description election in pt',
            'elections-0-description_en': 'description election in en',
            'elections-0-proposition_begin_date': '2016-10-26 15:00:00',
            'elections-0-proposition_end_date': '2016-10-27 15:00:00',
            'elections-0-roll_begin_date': '2016-10-28 15:00:00',
            'elections-0-roll_end_date': '2016-10-29 15:00:00',
            'elections-0-voting_begin_date': '2016-10-30 15:00:00',
            'elections-0-voting_end_date': '2016-11-01 15:00:00',
            'elections-0-voting_references_date': '2016-11-02 15:00:00',
            'elections-0-minute_publication_date': '2016-11-03 15:00:00',
            'elections-0-minute_begin_date': '2016-11-04 15:00:00',
            'elections-0-minute_end_date': '2016-11-05 15:00:00',
        }

    def test_get(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_elections.html")

    def test_post_valid(self):
        """ """
        response = self.client.post(self.url, data=self.data_valid)
        errors = True
        if 'errors_list' not in response._container[0]:
            errors = False
        self.assertEqual(response.status_code, 200)
        self.assertEqual(errors, False)

    def test_post_invalid(self):
        """ """
        data = {
            'elections-TOTAL_FORMS': '1',
            'elections-INITIAL_FORMS': '0',
            'elections-MIN_NUM_FORMS': '1',
            'elections-MAX_NUM_FORMS': '1000',
            'elections-0-identifier_pt': 'election test in pt',
            'elections-0-identifier_en': 'election test in en',
            'elections-0-description_pt': '',
            'elections-0-description_en': '',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_post_valid_as_process_new(self):
        """ """
        process = self.create_electoral_process(
            state=ElectoralProcess.State.NEW)
        self.create_election(electoral_process=process)
        url = reverse(
            "management:electoralprocess-elections", kwargs={
                "slug": process.slug
            })
        response = self.client.post(url, data=self.data_valid)
        errors = True
        if 'errors_list' not in response._container[0]:
            errors = False
        self.assertEqual(response.status_code, 200)
        self.assertEqual(errors, False)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectoralProcessMembersViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process()
        self.url = reverse(
            "management:electoralprocess-members", kwargs={
                "slug": self.process.slug
            })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_members.html")

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-members", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_as_member(self):
        self.client.logout()
        self.login_as_member()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_members.html")


def failed_delete(*args, **kwargs):
    return None, []


class ElectoralMemberRemoveViewTest(ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.member = self.create_electoral_member()
        self.process = self.member.electoral_process

    def test_get_ok(self):
        """ """
        url = reverse("management:electoralmember-remove", kwargs={
            "process_pk": self.process.pk,
            "pk": self.member.pk
        })
        valid_qs = ElectoralMember.objects.filter(pk=self.member.pk).count()
        response = self.client.get(url)
        members_qs = ElectoralMember.objects.filter(
            electoral_process=self.process).count()

        self.assertEqual(response.status_code, 302)
        self.assertEqual(valid_qs, 1)
        self.assertEqual(members_qs, 0)

    @patch('evotevotings.processes.models.ElectoralMember.delete',
           new=failed_delete)
    def test_get_not_removed(self):
        url = reverse("management:electoralmember-remove", kwargs={
            "process_pk": self.process.pk,
            "pk": self.member.pk
        })
        valid_qs = ElectoralMember.objects.filter(pk=self.member.pk).count()
        response = self.client.get(url)
        members_qs = ElectoralMember.objects.filter(
            electoral_process=self.process).count()

        self.assertEqual(response.status_code, 302)
        self.assertEqual(valid_qs, 1)
        self.assertEqual(members_qs, 1)


class ElectoralProcessSummaryViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process()
        self.member = self.create_electoral_member(
            electoral_process=self.process, role=ElectoralMember.Role.CHAIRMAN)
        self.election = self.create_election(electoral_process=self.process)
        self.url = reverse("management:electoralprocess-summary", kwargs={
            "slug": self.process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_summary.html")

    def test_redirect_when_no_chairman_associated(self):
        """ """
        self.member.role = ElectoralMember.Role.OTHER
        self.member.save()
        response = self.client.get(self.url)
        redirect_to = reverse("management:electoralprocess-members", kwargs={
            "slug": self.process.slug
        })
        self.assertRedirects(response, redirect_to)

    def test_redirect_when_no_members_associated(self):
        """ """
        self.member.delete()
        response = self.client.get(self.url)
        redirect_to = reverse("management:electoralprocess-members", kwargs={
            "slug": self.process.slug
        })
        self.assertRedirects(response, redirect_to)

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-summary", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectoralProcessConfirmationViewTest(ManagementTestCase,
                                           ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_institutional()
        self.process = self.create_electoral_process(state=4)
        self.election = self.create_election(electoral_process=self.process)
        self.member = self.create_electoral_member(
            electoral_process=self.process, role=ElectoralMember.Role.CHAIRMAN)
        self.url = reverse(
            "management:electoralprocess-confirmation", kwargs={
                "slug": self.process.slug
            })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_confirmation.html")

    def test_cannot_proceed(self):
        """ """
        process = self.create_electoral_process(electoral_members=None)
        url = reverse(
            "management:electoralprocess-confirmation", kwargs={
                "slug": process.slug
            })
        response = self.client.get(url)
        message = list(response.context['messages'])[0]
        self.assertEqual(response.status_code, 200)
        self.assertEqual(str(message), "Unable to submit.")

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-confirmation", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectoralProcessResponsibleViewTest(ManagementTestCase,
                                          ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.process = self.create_electoral_process(state=4)
        self.election = self.create_election(electoral_process=self.process)
        self.member = self.create_electoral_member(
            electoral_process=self.process, role=ElectoralMember.Role.CHAIRMAN,
            is_comunication_responsible=True)
        self.url = reverse(
            "management:electoralprocess-responsible", kwargs={
                "slug": self.process.slug
            })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/electoralprocess_responsible.html")

    def test_change_responsible(self):
        data = {
            'responsible-responsible_communication': self.member.pk
        }
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_with_unexistent_object(self):
        """ """
        url = reverse("management:electoralprocess-responsible", kwargs={
            "slug": "test"
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ElectionDetailConfigsViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election()
        self.url = reverse("management:election-detail-configs", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_configs.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_success_url(self):
        data = model_to_dict(self.election)
        # garantee that election has hondt_total_seats correctly filled
        data.update({
            'hondt_total_seats': 1
        })
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        success_url = reverse("management:election-detail-calendar", kwargs={
            'slug': self.election.electoral_process.slug,
            'slug_election': self.election.slug
        })
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], success_url)


class ElectionDetailCalendarViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election()
        self.url = reverse("management:election-detail-calendar", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_calendar.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_success_url(self):
        data = {
            'proposition_begin_date': '10-10-2016 10:00',
            'proposition_end_date': '11-10-2016 10:00',
            'roll_begin_date': '12-10-2016 10:00',
            'roll_end_date': '13-10-2016 10:00',
            'voting_begin_date': '14-10-2016 10:00',
            'voting_end_date': '15-10-2016 10:00',
            'voting_references_date': '15-10-2016 10:00',
            'minute_publication_date': '16-10-2016 10:00',
            'minute_begin_date': '17-10-2016 10:00',
            'minute_end_date': '18-10-2016 10:00'
        }
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        success_url = reverse("management:election-detail-categories", kwargs={
            'slug': self.election.electoral_process.slug,
            'slug_election': self.election.slug
        })
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], success_url)


class ElectionDetailCategoriesViewTest(ManagementTestCase, ElectionsTestUtils):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election()
        self.url = reverse("management:election-detail-categories", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_categories.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_success_url(self):
        data = {
            'categories-TOTAL_FORMS': '1',
            'categories-INITIAL_FORMS': '1',
            'categories-0-id': '1',
            'categories-0-election': self.election.pk,
            'categories-0-designation_pt': 'Um',
            'categories-0-designation_en': 'One',
            'categories-0-weight': '1'
        }
        response = self.client.post(self.url, data)
        content = json.loads(response.content)
        success_url = reverse("management:election-detail-rolls", kwargs={
            'slug': self.election.electoral_process.slug,
            'slug_election': self.election.slug
        })
        self.assertEqual(response.status_code, 200)
        self.assertEqual(content['action'], AjaxResponseAction.REDIRECT)
        self.assertEqual(content['action_url'], success_url)


class ElectionDetailCandidatesViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election()
        self.url = reverse("management:election-detail-candidates", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_candidates.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionDetailBulletinViewTest(ElectionsTestUtils, ManagementTestCase):
    """ """

    def setUp(self):
        self.login_as_member()
        self.election = self.create_election()
        self.url = reverse("management:election-detail-bulletin", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "management/election_detail_bulletin.html")

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class PersonalDataViewTest(ManagementTestCase):

    def setUp(self):
        self.login()
        self.url = reverse("management:personal-data")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'management/personal_data.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
