from __future__ import unicode_literals

from django.contrib.auth.decorators import login_required
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.base import View
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic import ListView, DetailView
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from django.utils import timezone

from evotevotings.elections.importers import CandidateImporter
from evotevotings.elections.models import Election, ElectoralRoll
from evotevotings.processes.models import ElectoralProcess, ElectoralMember
from evotevotings.processes.utils import prepopulate_electoralactions
from evotevotings.elections.forms import ElectoralCalendarForm
from evotevotings.elections.tasks import import_electoral_roll
from evoteajaxforms.views import (FormJsonView, FormsetJsonView,
                                  AjaxResponseAction)
from evoteusers.views import EvoteLoginView, PersonalDataBaseView
from django_tables2 import SingleTableMixin
from django_fsm import can_proceed

from .forms import (ManagementAuthenticationForm, ElectoralActionFormset,
                    ElectionDetailConfigsForm, ElectionFormset,
                    CategoryFormset, ElectoralRollDocumentForm,
                    ElectoralProcessAsMemberForm,
                    ElectoralProcessCommunicationForm,
                    ElectoralProcessAsInstitutionalForm,
                    ElectoralDocumentFormset, CandidateFormset,
                    ElectionWriteInDocumentForm, ElectionBulletinForm)

from .tables import (ElectoralProcessMemberTable, ElectoralProcessDetailTable,
                     ElectoralProcessInstitutionalTable, ElectionTable,
                     ElectoralMemberSimpleTable, ElectoralMemberTable,
                     ElectoralRollTable, ElectionVoterTable)


class LoginView(EvoteLoginView):
    """Provides user the ability to login"""
    form_class = ManagementAuthenticationForm
    template_name = "management/login.html"

    def form_valid(self, form):
        self.request.session['user_profile'] = form.cleaned_data.get('profile')
        return super(LoginView, self).form_valid(form)


"""
View electoral process's
"""


@method_decorator(login_required, name="dispatch")
class ElectoralProcessListView(SingleTableMixin, ListView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_list.html"

    table_pagination = {'per_page': 5}

    def get_queryset(self, **kwargs):
        """
        Returns the Electoral Process that are published with limit
        """
        if self.request.session['user_profile'] == \
                ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER:
            return ElectoralProcess.objects\
                .belongs_to_member(self.request.user).public()
        return ElectoralProcess.objects\
            .belongs_to_institutional_member(self.request.user)

    def get_table_class(self):
        """
        """
        if self.request.session['user_profile'] == \
                ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER:
            return ElectoralProcessMemberTable
        return ElectoralProcessInstitutionalTable


@method_decorator(login_required, name="dispatch")
class ElectoralProcessDetailView(SingleTableMixin, DetailView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_detail.html"

    table_class = ElectoralProcessDetailTable
    table_pagination = False

    def get_table_data(self):
        return [self.object]


@method_decorator(login_required, name="dispatch")
class ElectoralActionView(DetailView):
    """ """
    queryset = ElectoralProcess.objects.all()
    formset = ElectoralActionFormset
    template_name = "management/electoralprocess_actions.html"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        prepopulate_electoralactions(self.object)
        formset = self.formset(
            request=self.request, instance=self.object, action="manage")
        context = self.get_context_data(object=self.object, formset=formset)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        """ """
        self.object = self.get_object()
        formset = self.formset(
            request.POST, request=self.request, instance=self.object,
            action="manage")
        if formset.is_valid():
            formset.save()
            # update flag to indicates actions have been successfully
            # configurated
            if not self.object.actions_configured:
                self.object.actions_configured = True
                self.object.save(update_fields=["actions_configured"])
            messages.success(request, _("Operations updated successfully"))
        else:
            messages.error(request, _("Unable to update the operations"))
        context = self.get_context_data(object=self.object, formset=formset)
        return self.render_to_response(context)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessFormView(FormJsonView):
    """ """
    model = ElectoralProcess
    prefix = 'process'
    template_name = "management/electoralprocess_form.html"
    formset = ElectoralDocumentFormset
    formset_prefix = 'electoral-documents'

    def get_form_class(self):
        if self.request.session['user_profile'] == \
                ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER:
            return ElectoralProcessAsMemberForm
        return ElectoralProcessAsInstitutionalForm

    def form_valid(self, form):
        is_formset_valid = True
        documents_formset = None
        if self.request.session['user_profile'] == \
                ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER:
                documents_formset = self.formset(
                    self.request.POST, self.request.FILES,
                    prefix=self.formset_prefix, instance=self.object)
                if not documents_formset.is_valid():
                    is_formset_valid = False
        if is_formset_valid:
            process = form.save(commit=False)
            process.created_by = self.request.user
            process.save()
            if documents_formset:
                documents_formset.save()
            if not self.object or process.is_draft():
                self.success_url = reverse_lazy(
                    "management:electoralprocess-elections",
                    args=[process.slug])
            else:
                self.action = AjaxResponseAction.REFRESH
            messages.success(
                self.request, _("Operations executed successfully."))
            return self.json_to_response()
        else:
            self.action = AjaxResponseAction.REFRESH
            return self.formset_invalid(documents_formset, self.formset_prefix)

    def get_object(self):
        if self.kwargs.get('slug', None):
            return self.model.objects.get(slug=self.kwargs.get('slug'))
        return None

    def get_context_data(self, **kwargs):
        """ """
        context = super(ElectoralProcessFormView, self)\
            .get_context_data(**kwargs)
        context.update({
            'new_state': ElectoralProcess.State.NEW,
        })
        if self.request.session['user_profile'] == \
                ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER:
            context.update({
                'docformset': self.formset(
                    prefix=self.formset_prefix, instance=self.object)
            })
        return context


@method_decorator(login_required, name="dispatch")
class ElectoralProcessElectionsView(FormsetJsonView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_elections.html"
    form_class = ElectionFormset
    prefix = 'elections'

    def form_valid(self, form):
        if self.object.is_draft():
            self.success_url = reverse_lazy(
                "management:electoralprocess-members",
                args=[self.object.slug])
            self.action = AjaxResponseAction.REDIRECT
        else:
            self.action = AjaxResponseAction.REFRESH
        messages.success(
            self.request, _("Operations executed successfully."))
        return super(ElectoralProcessElectionsView, self).form_valid(form)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessMembersView(SingleTableMixin, ListView):
    """ """
    model = ElectoralMember
    template_name = "management/electoralprocess_members.html"

    def get(self, request, *args, **kwargs):
        """ """
        self.object = get_object_or_404(ElectoralProcess,
                                        slug=self.kwargs['slug'])
        return super(ElectoralProcessMembersView, self).get(
            request, *args, **kwargs)

    def get_table_class(self):
        allow_edit_in = [ElectoralProcess.State.NEW,
                         ElectoralProcess.State.DRAFT]
        if self.request.session['user_profile'] == \
                ManagementAuthenticationForm.Profiles.ELECTORAL_MEMBER or\
                self.object.state not in allow_edit_in:
            return ElectoralMemberSimpleTable
        return ElectoralMemberTable

    def get_queryset(self):
        return self.model.objects.filter(
            electoral_process__slug=self.kwargs.get('slug'))

    def get_context_data(self, **kwargs):
        context = super(ElectoralProcessMembersView, self)\
            .get_context_data(**kwargs)
        context.update({
            "object": self.object
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectoralMemberRemoveView(View):

    def get(self, request, *args, **kwargs):
        process_pk = kwargs.get('process_pk', None)
        member_pk = kwargs.get('pk', None)
        member = get_object_or_404(
            ElectoralMember, pk=member_pk,
            electoral_process_id=process_pk)
        process = member.electoral_process
        deleted, obj_list = member.delete()

        if deleted and obj_list:
            messages.success(request, _("Electoral member was deleted"))
        else:
            messages.error(
                request,
                _("There was a problem removing the electoral member"))

        return redirect('management:electoralprocess-members',
                        slug=process.slug)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessSummaryView(SingleTableMixin, DetailView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_summary.html"

    table_class = ElectoralMemberSimpleTable
    table_pagination = False

    def get_table_data(self):
        return self.object.electoral_members.all()

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        if not self.object.electoral_members.exists():
            messages.error(request, _("No members inserted!"))
            return redirect('management:electoralprocess-members',
                            slug=self.kwargs['slug'])
        elif not self.object.electoral_members.all().filter(
                role=ElectoralMember.Role.CHAIRMAN).exists():
            messages.error(
                request, _("No chairman inserted! Please insert one."))
            return redirect('management:electoralprocess-members',
                            slug=self.kwargs['slug'])
        return super(ElectoralProcessSummaryView, self)\
            .dispatch(request, *args, **kwargs)


@method_decorator(login_required, name="dispatch")
class ElectoralProcessConfirmationView(DetailView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_confirmation.html"

    def get(self, request, *args, **kwargs):
        response = super(ElectoralProcessConfirmationView, self).get(
            request, *args, **kwargs)
        if not can_proceed(self.object.submit_new):
            messages.error(request, _("Unable to submit."))
            return response
        chairmans = ElectoralMember.objects.filter(
            electoral_process=self.object, role=ElectoralMember.Role.CHAIRMAN)
        if chairmans.exists():
            ElectoralMember.objects.mark_as_responsible(
                self.object, chairmans.first())
        self.object.submit_new()
        self.object.save()
        return response


@method_decorator(login_required, name="dispatch")
class ElectoralProcessResponsibleView(FormJsonView):
    """ """
    model = ElectoralProcess
    template_name = "management/electoralprocess_responsible.html"
    form_class = ElectoralProcessCommunicationForm
    prefix = 'responsible'
    action = AjaxResponseAction.REFRESH

    def get_form_kwargs(self):
        form_kwargs = super(ElectoralProcessResponsibleView, self)\
            .get_form_kwargs()
        chairmans = ElectoralMember.objects.filter(
            electoral_process=self.object,
            is_comunication_responsible=True)
        if chairmans.exists():
            form_kwargs.update({
                'initial': {
                    'responsible_communication': chairmans.first()
                }
            })
        return form_kwargs

    def form_valid(self, form):
        member = form.cleaned_data['responsible_communication']
        ElectoralMember.objects.mark_as_responsible(self.object, member)
        messages.success(
            self.request, _("Responsible of communication updated."))
        return super(ElectoralProcessResponsibleView, self).form_valid(
            form)


"""
Views elections
"""


@method_decorator(login_required, name="dispatch")
class ElectionListView(SingleTableMixin, ListView):
    """ """
    model = Election
    table_class = ElectionTable
    template_name = "management/election_list.html"

    def get_queryset(self):
        return self.model.objects.filter(
            electoral_process__slug=self.kwargs.get('slug'))\
            .select_related('electoral_process')

    def get_context_data(self, **kwargs):
        context = super(ElectionListView, self).get_context_data(**kwargs)
        context.update({
            "electoral_process": self.get_queryset().first().electoral_process
        })
        return context


@method_decorator(login_required, name="dispatch")
class ElectionDetailConfigsView(FormJsonView):
    """ """
    model = Election
    form_class = ElectionDetailConfigsForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_configs.html"

    def form_valid(self, form):
        self.object.configs_filled = True
        self.object.save(update_fields=["configs_filled"])
        return super(ElectionDetailConfigsView, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy("management:election-detail-calendar", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailCalendarView(FormJsonView):
    """ """
    model = Election
    form_class = ElectoralCalendarForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_calendar.html"

    def form_valid(self, form):
        self.object.calendar_filled = True
        self.object.save(update_fields=["calendar_filled"])
        return super(ElectionDetailCalendarView, self).form_valid(form)

    def get_success_url(self):
        return reverse_lazy("management:election-detail-categories", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailCategoriesView(FormsetJsonView):
    """ """
    model = Election
    prefix = "categories"
    form_class = CategoryFormset
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_categories.html"

    def get_success_url(self):
        return reverse_lazy("management:election-detail-rolls", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailRollsView(SingleTableMixin, FormJsonView):
    """ """
    model = Election
    action = AjaxResponseAction.REFRESH
    form_class = ElectoralRollDocumentForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_rolls.html"

    table_class = ElectoralRollTable
    table_pagination = {'per_page': 15}

    def form_valid(self, form):
        """ """
        document = form.save()
        electoral_roll = ElectoralRoll.objects.create(
            election=self.object, document=document)
        import_electoral_roll.delay(electoral_roll.pk)
        return self.json_to_response()

    def get_form_kwargs(self):
        kwargs = super(ElectionDetailRollsView, self).get_form_kwargs()
        kwargs.update({
            "election": self.object,
            "is_importing": ElectoralRoll.objects.is_importing(self.object.pk)
        })
        return kwargs

    def get_table_data(self):
        return self.object.electoral_rolls.all()


@method_decorator(login_required, name="dispatch")
class ElectoralRollPublishView(SingleObjectMixin, View):
    """ """
    model = ElectoralRoll

    def post(self, request, *args, **kwargs):
        electoral_roll = self.get_object()
        electoral_roll.published_on = timezone.now()
        electoral_roll.save(update_fields=['published_on'])
        return redirect('management:election-detail-rolls',
                        slug=electoral_roll.election.electoral_process.slug,
                        slug_election=electoral_roll.election.slug)


@method_decorator(login_required, name="dispatch")
class ElectionDetailCandidatesView(FormJsonView):
    """ """
    model = Election
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_candidates.html"

    def get_prefix(self):
        """ """
        if not self.object.write_in:
            return "candidates"
        return "document"

    def get_form_class(self):
        """ """
        if not self.object.write_in:
            return CandidateFormset
        return ElectionWriteInDocumentForm

    def get_form_kwargs(self):
        """ """
        kwargs = super(ElectionDetailCandidatesView, self).get_form_kwargs()
        if self.object.write_in:
            kwargs.update({
                "election": self.object,
                "instance": self.object.write_in_import
            })
        else:
            kwargs.update({
                "queryset":
                    self.object.candidates.not_write_in().order_by('created')
            })
        return kwargs

    def form_valid(self, form):
        """ """
        instance = form.save()
        if self.object.write_in:
            if not self.object.write_in_import:
                self.object.write_in_import = instance
                self.object.save()
            importer = CandidateImporter(self.object)
            importer.run(instance.document)
        return self.json_to_response()

    def form_invalid(self, form, prefix=None):
        """ """
        if not self.object.write_in:
            return self.formset_invalid(form, prefix)
        return super(ElectionDetailCandidatesView, self)\
            .form_invalid(form, prefix)

    def get_success_url(self):
        return reverse_lazy("management:election-detail-bulletin", kwargs={
            'slug': self.kwargs.get('slug'),
            'slug_election': self.kwargs.get('slug_election')
        })


@method_decorator(login_required, name="dispatch")
class ElectionDetailBulletinView(FormJsonView):
    """ """
    model = Election
    prefix = "bulletin"
    form_class = ElectionBulletinForm
    slug_url_kwarg = "slug_election"
    template_name = "management/election_detail_bulletin.html"

    def get_success_url(self):
        return reverse_lazy("management:election-list", kwargs={
            'slug': self.kwargs.get('slug'),
        })


@method_decorator(login_required, name="dispatch")
class ElectoralRollDetailView(SingleTableMixin, DetailView):
    """ """
    model = ElectoralRoll
    table_class = ElectionVoterTable
    template_name = "management/electoralroll_detail.html"

    def get_table_data(self):
        """ """
        return self.object.election_voters.all()


@method_decorator(login_required, name="dispatch")
class PersonalDataView(PersonalDataBaseView):
    """ """
    template_name = "management/personal_data.html"
