from __future__ import unicode_literals

from django.db.models.signals import post_save, post_init
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible
from django.utils import timezone
from django.conf import settings
from django.db import models

from phonenumber_field.modelfields import PhoneNumberField
from django_extensions.db.models import TimeStampedModel
from dateutil.relativedelta import relativedelta
from evotecore.mixins import ModelFieldChangedMixin
from evotedocs.models import Document
from evotecore.utils import images_upload_to
from sorl.thumbnail import ImageField
from django_fsm import FSMIntegerField, transition
from autoslug import AutoSlugField
from evotemailings.mixins import MailingListMixin

from .managers import ElectoralProcessQuerySet, ElectoralMemberManager
from .tasks import publish_electoral_process


@python_2_unicode_compatible
class Action(TimeStampedModel):
    """ """
    designation = models.CharField(_("action"), max_length=128, db_index=True)

    class Meta:
        verbose_name = _("action")
        verbose_name_plural = _("actions")

    def __str__(self):
        return self.designation


@python_2_unicode_compatible
class ElectoralProcess(
        MailingListMixin, ModelFieldChangedMixin, TimeStampedModel):
    """ """

    class State:
        NEW = 0
        OPEN = 1
        CLOSED = 2
        ARCHIVED = 3
        DRAFT = 4

        choices = (
            (NEW, _("New")),
            (OPEN, _("Open")),
            (CLOSED, _("Closed")),
            (ARCHIVED, _("Archived")),
            (DRAFT, _("Draft")),
        )

    logo = ImageField(_('Logo (minimum size 135px x 100px)'),
                      upload_to=images_upload_to, blank=True)
    slug = AutoSlugField(
        populate_from='identifier', unique=True, editable=False)
    identifier = models.CharField(
        _("electoral process identifier"), max_length=128, db_index=True,
        help_text=_("Insert the electoral process designation"))
    description = models.CharField(
        _("electoral process description"), max_length=512,
        help_text=_("brief description text about electoral process"))
    state = FSMIntegerField(
        _("state"), choices=State.choices, default=State.DRAFT)
    closed_on = models.DateTimeField(_("closed on"), blank=True, null=True)
    actions = models.ManyToManyField(Action, through="ElectoralAction",
                                     verbose_name=_("actions"), blank=True)
    actions_configured = models.BooleanField(default=False, editable=False)
    publication_date = models.DateTimeField(_("publication date"), blank=True,
                                            null=True)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_("created by"),
        on_delete=models.CASCADE, related_name="electoral_processes")
    objects = ElectoralProcessQuerySet.as_manager()

    class Meta:
        ordering = ['-publication_date']
        verbose_name = _("electoral process")
        verbose_name_plural = _("electoral processes")

    def __str__(self):
        return self.identifier

    def can_publish(self):
        """Check if publication date has been reached, otherwise cant publish
        """
        return self.publication_date <= timezone.now()

    def can_close(self):
        """Check if all elections are closed, otherwise cant close
        """
        return self.elections.closed().count() == self.elections.count()

    def can_archive(self):
        """Check if EVOTE_ARCHIVE_PROCESS_AFTER period has been reached,
        otherwise cant archive"""
        diff = timezone.now() - \
            relativedelta(months=+settings.EVOTE_ARCHIVE_PROCESS_AFTER)
        return self.closed_on <= diff

    def can_submit(self):
        """Check if electoral process has members"""
        return self.electoral_members.exists()

    @transition("state", source=State.DRAFT, target=State.NEW,
                conditions=[can_submit])
    def submit_new(self):
        """
        Submit new electoral process
        """
        # TODO: notify all electoral members
        pass

    @transition("state", source=State.NEW, target=State.OPEN,
                conditions=[can_publish])
    def publish(self):
        """Publish the electoral process when publication_date is reached and
        all associated elections
        """
        for election in self.elections.all():
            election.publish()
            election.save()

    @transition("state", source=State.OPEN, target=State.CLOSED,
                conditions=[can_close])
    def close(self):
        """Transition only called after all his elections are closed"""
        self.closed_on = timezone.now()

    @transition("state", source=State.CLOSED, target=State.ARCHIVED,
                conditions=[can_archive])
    def archive(self):
        """Delete all votes from db (counter service) and all data from
        validator service"""
        pass

    def is_draft(self):
        """
        Check if electoral process is in state draft
        """
        return self.state == self.State.DRAFT

    def is_open(self):
        """
        Check if electoral process is in state open
        """
        return self.state == self.State.OPEN

    def get_mailing_list(self):
        """ """
        emails = []
        for member in self.electoral_members.all():
            emails.append(
                (member.user.get_full_name(), member.user.get_email()))
        return emails

    @staticmethod
    def post_init(sender, instance, **kwargs):
        """Callback staticmethod for post_init signal"""
        instance.track_changes(['publication_date'])

    @staticmethod
    def post_save(sender, instance, **kwargs):
        """Callback staticmethod for post_save signal"""
        if instance.has_changed('publication_date') and not instance.closed_on:
            publish_electoral_process.si(
                instance.pk).apply_async(eta=instance.publication_date)


post_init.connect(ElectoralProcess.post_init, sender=ElectoralProcess)
post_save.connect(ElectoralProcess.post_save, sender=ElectoralProcess)


@python_2_unicode_compatible
class ElectoralAction(TimeStampedModel):
    """ """
    class Strength:
        SIMPLE = 0
        STRONG = 1
        CRITICAL = 2

        choices = (
            (SIMPLE, _("Simple")),
            (STRONG, _("Strong")),
            (CRITICAL, _("Critical")),
        )
    action = models.ForeignKey(Action, verbose_name=_("action"),
                               on_delete=models.CASCADE,
                               related_name="electoral_actions")
    electoral_process = models.ForeignKey(
        ElectoralProcess, verbose_name=_("electoral process"),
        on_delete=models.CASCADE, related_name="electoral_actions")
    strength = models.PositiveSmallIntegerField(
        _("strength"), choices=Strength.choices, blank=True,
        default=Strength.SIMPLE)

    class Meta:
        verbose_name = _("electoral action")
        verbose_name_plural = _("electoral actions")

    def __str__(self):
        return "{} ({})".format(self.action, self.get_strength_display())


@python_2_unicode_compatible
class ElectoralMember(TimeStampedModel):
    """ """

    class Role:
        CHAIRMAN = 0
        OTHER = 1

        choices = (
            (CHAIRMAN, _("Chairman")),
            (OTHER, _("Other")),
        )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_("user"),
        on_delete=models.CASCADE, related_name="electoral_members")
    electoral_process = models.ForeignKey(
        ElectoralProcess, verbose_name=_("electoral process"),
        related_name="electoral_members", on_delete=models.CASCADE)
    designation = models.CharField(_("designation"), max_length=128,
                                   db_index=True)
    role = models.PositiveSmallIntegerField(_("role"), choices=Role.choices)
    phone = PhoneNumberField(_("contact phone number"), blank=True)
    private_key = models.BinaryField(_("key"), blank=True)
    is_comunication_responsible = models.BooleanField(default=False)
    objects = ElectoralMemberManager()

    class Meta:
        verbose_name = _("electoral member")
        verbose_name_plural = _("electoral members")

    def __str__(self):
        return self.user.name + " (" + self.get_role_display() + ")"

    @property
    def has_private_key(self):
        return bool(self.private_key)


@python_2_unicode_compatible
class ElectoralDocument(TimeStampedModel):
    """ """
    electoral_process = models.ForeignKey(
        ElectoralProcess, verbose_name=_("electoral process"),
        related_name="electoral_documents", on_delete=models.CASCADE)
    designation = models.CharField(_("designation"), max_length=128,
                                   db_index=True)
    documents = models.ManyToManyField(Document, verbose_name=_("documents"),
                                       blank=True)

    class Meta:
        verbose_name = _("Electoral document")
        verbose_name_plural = _("Electoral documents")

    def __str__(self):
        return self.designation
