from __future__ import unicode_literals

from dateutil.relativedelta import relativedelta
from unittest import skip

from django.utils import timezone
from django.test import SimpleTestCase, TestCase
from django.conf import settings

from evotevotings.elections.tests.utils import ElectionsTestUtils
from evotevotings.elections.models import Election
from evotevotings.processes.models import (Action, ElectoralProcess,
                                           ElectoralAction, ElectoralMember,
                                           ElectoralDocument)
from evoteusers.models import User

from .utils import ProcessesTestUtils


class ActionTest(SimpleTestCase):
    """ """

    def test_str_equal_to_designation(self):
        instance = Action(designation="designation")
        self.assertEqual(str(instance), instance.designation)


class ElectoralProcessTest(ProcessesTestUtils, ElectionsTestUtils, TestCase):
    """ """

    def test_str_equal_to_identifier(self):
        instance = ElectoralProcess(identifier="identifier")
        self.assertEqual(str(instance), instance.identifier)

    def test_can_publish_condition(self):
        published_process = ElectoralProcess(
            publication_date=timezone.now() - relativedelta(hours=+1))
        unpublished_process = ElectoralProcess(
            publication_date=timezone.now() + relativedelta(hours=+1))
        self.assertTrue(published_process.can_publish())
        self.assertFalse(unpublished_process.can_publish())

    def test_can_close_condition(self):
        process = self.create_electoral_process()
        elections = self.create_elections(
            2, electoral_process=process, state=Election.State.CLOSED)
        self.assertTrue(process.can_close())
        elections[0].state = Election.State.OPEN
        elections[0].save()
        self.assertFalse(process.can_close())

    def test_can_archive_condition(self):
        process_archivable = ElectoralProcess(closed_on=timezone.now())
        self.assertFalse(process_archivable.can_archive())
        process_not_archivable = ElectoralProcess(
            closed_on=timezone.now() +
            relativedelta(months=+settings.EVOTE_ARCHIVE_PROCESS_AFTER))
        self.assertFalse(process_not_archivable.can_archive())

    def test_can_submit_condition(self):
        unsubmitable_process = self.create_electoral_process()
        self.assertFalse(unsubmitable_process.can_submit())
        submitable_process = self.create_electoral_member().electoral_process
        self.assertTrue(submitable_process.can_submit())

    @skip("to implement")
    def test_submit_new_transition(self):
        pass

    def test_publish_transition(self):
        process = self.create_electoral_process(
            state=ElectoralProcess.State.NEW,
            publication_date=timezone.now() - relativedelta(hours=+1))
        election = self.create_election(electoral_process=process)
        self.assertEqual(election.state, Election.State.NEW)
        process.publish()
        self.assertEqual(process.state, Election.State.OPEN)
        election.refresh_from_db()
        self.assertEqual(election.state, Election.State.OPEN)

    def test_close_transition(self):
        process = ElectoralProcess(state=ElectoralProcess.State.OPEN)
        process.close()
        self.assertIsNotNone(process.closed_on)

    @skip("to implement")
    def test_archive_transition(self):
        pass


class ElectoralActionTest(SimpleTestCase):
    """ """

    def test_str_equal_to_action_and_strength(self):
        electoral_action = ElectoralAction(
            action=Action(designation="designation"),
            strength=ElectoralAction.Strength.SIMPLE)
        text = "{} ({})".format(
            electoral_action.action, electoral_action.get_strength_display())
        self.assertEqual(str(electoral_action), text)


class ElectoralMemberTest(SimpleTestCase):
    """ """

    def test_has_private_key(self):
        instance = ElectoralMember()
        self.assertFalse(instance.has_private_key)
        instance.private_key = b"Dummy content"
        self.assertTrue(instance.has_private_key)

    def test_str_equal_to_designation(self):
        user = User(name="Dev")
        instance = ElectoralMember(user=user, role=0)
        expected = instance.user.name + \
            " (" + instance.get_role_display() + ")"
        self.assertEqual(str(instance), expected)


class ElectoralDocumentTest(SimpleTestCase):
    """ """

    def test_str_equal_to_designation(self):
        instance = ElectoralDocument(designation="designation")
        self.assertEqual(str(instance), instance.designation)
