from rest_framework.response import Response
from rest_framework.generics import CreateAPIView
from rest_framework.renderers import JSONRenderer
import requests
import json
from django.conf import settings
from eVotUM.Cripto import (pkiutils,
                           eccblind)
from . import serializers
from evotevotings.services.validator import utils
from evotevotings.elections.utils import (get_certificates,
                                          validate_certificate)


class FilterSignatureView(CreateAPIView):
    """
    Standard API view
    """
    permission_classes = []
    serializer_class = serializers.FilterSignatureSerializer

    def get_serializer_class(self):
        """
        Define the Serializer class to be interpreted
        by Django Swagger
        """
        return self.serializer_class

    def post(self, request, *args, **kwargs):
        """
        POST method docs
        Returns either true or false
        """
        serializer = serializers.FilterSignatureSerializer(data=request.data)
        # Top-level validation through the serializer
        serializer.is_valid(raise_exception=True)
        sanitized_data = serializer.validated_data
        response_dict = {"result": "false"}
        # Validate the request data
        try:
            # Get the necessary certificates
            certs = get_certificates()

            # Missing voting system certificate
            if (not certs['votesys']):
                raise Exception('Missing Voting System Certificate')

            # Missing root certificate
            if (not certs['root']):
                raise Exception('Missing Root Certificate')

            # Missing Validator Service certificate
            if (not certs['vs']):
                raise Exception('Missing Validator Service Certificate')

            # Check if Voting System certificate is chained to Root
            valid = validate_certificate(certs['votesys'], certs['root'])

            # Not a chained certificate
            if (valid == -1):
                raise Exception('Unchained Voting System certificate')
            # Expired certificate
            if (valid == -2):
                raise Exception('Expired Voting System certificate')

            # Validate signed election_id object
            signed_obj = str(sanitized_data['election_id'])
            (error_code, election_id) = pkiutils.verifyObjectSignature(
                                            signed_obj,
                                            certs['votesys'])

            if(error_code is not None):
                raise Exception('('+str(error_code) +
                                '): Invalid election+user signature')

            # Validate vote object blind signature
            (error_code, unsigned_vote) = eccblind.verifySignature(
                                            certs['vs_pub'],
                                            sanitized_data['signed_vote'],
                                            sanitized_data['blind_components'],
                                            sanitized_data['pr_components'],
                                            sanitized_data['cyphered_vote'])

            if(error_code is not None):
                raise Exception('('+str(error_code) +
                                '): Invalid vote blind signature')

            # Check if vote is available for this user
            return_data = utils.check_vote_status(
                                                sanitized_data['blind_hash'],
                                                election_id
                                                )
            if(not return_data):
                raise Exception('Vote not found for the given blind hash')

            # Remove the hash from the dictionary and send the
            # remaining fields to the anonymizer service
            sanitized_data.pop('blind_hash')

            # Replace the signed election_id with the unsigned value
            sanitized_data['election_id'] = election_id

            # Send to the Anonymizer Service
            vote_endpoint = settings.EVOTE_ANONYMIZER_INSERT_VOTE
            if(return_data and vote_endpoint):
                return_data = requests.post(vote_endpoint, sanitized_data)

            # If the response is not true, revert the vote status
            response_dict['result'] = json.dumps(return_data)
            return Response(JSONRenderer().render(response_dict))

        except Exception as e:
            print e
            # TODO: Log
            return Response(JSONRenderer().render(response_dict))
