from __future__ import unicode_literals

import json

from unittest import skip

from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.test import TestCase, SimpleTestCase
from django.conf import settings
from mock import patch

from evotevotings.processes.tests.utils import ProcessesTestUtils
from evotevotings.elections.tests.utils import ElectionsTestUtils
from evoteusers.tests.utils import UserTestUtils


class LoginViewTest(UserTestUtils, TestCase):
    """ """

    def setUp(self):
        self.credentials = {'email': 'dev@eurotux.com', 'password': 'password'}
        self.user = self.create_user(**self.credentials)

    def test_used_template(self):
        """ """
        response = self.client.get(reverse('voters:login'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/login.html')

    def test_login_valid_user(self):
        """ """
        response = self.client.post(reverse('voters:login'), follow=True,
                                    data=self.credentials)
        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL)
        self.assertEqual(response.context[0]['user'], self.user)

    def test_login_with_wrong_credentials(self):
        self.credentials['password'] = "wrong_password"
        response = self.client.post(reverse('voters:login'),
                                    data=self.credentials)
        error_message = "Please enter a correct email and password. " +\
            "Note that both fields may be case-sensitive."
        self.assertFormError(response, "form", None, error_message)


class LogoutViewTest(UserTestUtils, TestCase):

    def setUp(self):
        self.login()

    def test_logout_view(self):
        """ """
        response = self.client.get(reverse('voters:logout'), follow=True)
        self.assertRedirects(response, settings.LOGIN_URL)
        self.assertIsInstance(response.context[0]['user'], AnonymousUser)


class ElectoralProcessListViewTest(UserTestUtils, TestCase):

    def setUp(self):
        # create and login user
        self.login()
        self.url = reverse("voters:electoralprocess-list")

    def test_used_template_to_unauthenticated(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "voters/electoralprocess_list.html")

    def test_used_template_to_authenticated(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_mylist.html")

    @skip("to implement")
    def test_context_table_class_to_unauthenticated(self):
        pass

    @skip("to implement")
    def test_context_table_class_to_authenticated(self):
        pass

    @skip("to implement")
    def test_context_queryset_to_unauthenticated(self):
        pass

    @skip("to implement")
    def test_context_queryset_to_authenticated(self):
        pass


class ElectoralProcessDetailViewTest(ProcessesTestUtils, TestCase):

    def setUp(self):
        self.process = self.create_electoral_process()
        self.url = reverse("voters:electoralprocess-detail", kwargs={
            "slug": self.process.slug
        })

    def test_used_template_to_unauthenticated(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_detail.html")

    def test_used_template_to_authenticated(self):
        """ """
        self.login()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_mydetail.html")


class ElectoralProcessArchiveViewTest(TestCase):

    def test_used_template(self):
        """ """
        response = self.client.get(reverse("voters:electoralprocess-archive"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "voters/electoralprocess_archive.html")

    @skip("to implement")
    def test_context_table_class(self):
        """TODO: implement test"""
        pass

    @skip("to implement")
    def test_context_queryset(self):
        """TODO: implement test"""
        pass


class ElectionHistoryViewTest(UserTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.url = reverse("voters:election-history")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "voters/election_history.html")

    def test_get_to_unauthenticated_user(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    @skip("to implement")
    def test_context_table_class(self):
        """TODO: implement test"""
        pass

    @skip("to implement")
    def test_context_queryset(self):
        """TODO: implement test"""
        pass


class ElectoralProcessVotingViewTest(ProcessesTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.process = self.create_electoral_process()
        self.url = reverse("voters:electoralprocess-voting", kwargs={
            "slug": self.process.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, 'voters/electoralprocess_voting.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep1ViewTest(UserTestUtils, ElectionsTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.election = self.create_election()
        self.url = reverse("voters:election-voting-step1", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step1.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_invalid_context_data(self):
        """ """
        response = self.client.get(self.url)
        context = response.context[-1]
        self.assertIn("is_valid", context)
        self.assertFalse(context['is_valid'])

    @patch("evotevotings.voters.views.get_extra_election_data")
    def test_context_data(self, mock_fn):
        """ """
        mock_fn.return_value = {
            'combo_id': 'dummy',
            'vs_cert': 'dummy',
            'election_id': 'dummy'
        }

        response = self.client.get(self.url)
        context = response.context[-1]
        self.assertIn("is_valid", context)
        self.assertTrue(context['is_valid'])


class ElectionVotingStep2ViewTest(UserTestUtils, ElectionsTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.election = self.create_election()
        self.url = reverse("voters:election-voting-step2", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step2.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep3ViewTest(UserTestUtils, ElectionsTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.election = self.create_election()
        self.url = reverse("voters:election-voting-step3", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step3.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep3PinViewTest(UserTestUtils, ElectionsTestUtils,
                                     TestCase):

    def setUp(self):
        self.login()
        self.election = self.create_election()
        self.url = reverse("voters:election-voting-step3-pin", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                'voters/election_voting_step3_pin.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionVotingStep4ViewTest(UserTestUtils, ElectionsTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.election = self.create_election()
        self.url = reverse("voters:election-voting-step4", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_voting_step4.html')

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)


class ElectionSearchVoteViewTest(UserTestUtils, ElectionsTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.election = self.create_election()
        self.url = reverse("voters:election-search-vote", kwargs={
            "slug": self.election.electoral_process.slug,
            "slug_election": self.election.slug
        })

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/election_search_vote.html')


class ContactsViewTest(SimpleTestCase):

    def setUp(self):
        self.url = reverse("voters:contacts")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/contacts.html')


class PersonalDataViewTest(UserTestUtils, TestCase):

    def setUp(self):
        self.login()
        self.url = reverse("voters:personal-data")

    def test_used_template(self):
        """ """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'voters/personal_data.html')

    def test_form_submit_valid(self):
        data = {
            'personal-photo': 'Photo',
            'personal-name': 'Name',
            'personal-email': 'test@test.com',
            'personal-primary_phone': '+351911111111',
            'personal-notice_method': 1,
            'personal-notice_phone': '',
            'personal-notice_email': 'test@test.com',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('errors_list', content)

    def test_form_submit_fail_mail(self):
        data = {
            'personal-photo': 'Photo',
            'personal-name': 'Name',
            'personal-email': 'test@test.com',
            'personal-primary_phone': '+351911111111',
            'personal-notice_method': 1,
            'personal-notice_phone': '',
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_form_submit_fail_phone(self):
        data = {
            'personal-photo': 'Photo',
            'personal-name': 'Name',
            'personal-email': 'test@test.com',
            'personal-primary_phone': '+351911111111',
            'personal-notice_method': 0,
        }
        response = self.client.post(self.url, data=data)
        content = json.loads(response.content)
        self.assertEqual(response.status_code, 200)
        self.assertIn('errors_list', content)

    def test_login_required(self):
        """ """
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)
